observer

package
v1.2.0 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: May 5, 2026 License: BSD-2-Clause Imports: 2 Imported by: 0

README

observer

Package observer contains a base set of observers to observe any update events.

See the main repository README.md for build and documentation details.

Documentation

Overview

Package observer implements an observer for Tarantool 3.0.

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

This section is empty.

Types

type Accumulator

type Accumulator struct {
	// contains filtered or unexported fields
}

Accumulator accumulates events and sends them by batches. It helps not to delay a subscription with a slow observer. While one batch is being sent, the other is accumulating.

func NewAccumulator

func NewAccumulator(observer discovery.Observer) *Accumulator

NewAccumulator creates an Observer with given inner observer.

func (*Accumulator) Observe

func (o *Accumulator) Observe(events []discovery.Event, err error)

Observe adds events to the event storage. In case of an error, Accumulator will send all stored events with the error.

Jump to