Documentation
¶
Overview ¶
Package filter implements filters for Tarantool 3.0 discovery.
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type GroupOneOf ¶
type GroupOneOf struct {
// Groups is a set of allowed groups.
Groups []string
}
GroupOneOf matches instances that have a group in the set.
type LabelsContain ¶
type LabelsContain struct {
// Labels is a set of application tags to pass the match.
Labels map[string]string
}
LabelsContain matches instances that have all application labels from the map with the same values.
type NameOneOf ¶
type NameOneOf struct {
// Names is a set of allowed names.
Names []string
}
NameOneOf matches instances that have a name in the set.
type ReplicasetOneOf ¶
type ReplicasetOneOf struct {
// Replicasets is a set of allowed replicasets.
Replicasets []string
}
ReplicasetOneOf matches instances that have a replicaset in the set.
type RolesContain ¶
type RolesContain struct {
// Roles is a set of roles to pass the match.
Roles []string
}
RolesContain matches instances that have all roles from the set.
Click to show internal directories.
Click to hide internal directories.