Documentation
¶
Overview ¶
Package observer implements an observer for Tarantool 3.0.
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type Accumulator ¶
type Accumulator struct {
// contains filtered or unexported fields
}
Accumulator accumulates events and sends them by batches. It helps not to delay a subscription with a slow observer. While one batch is being sent, the other is accumulating.
func NewAccumulator ¶
func NewAccumulator(observer discovery.Observer) *Accumulator
NewAccumulator creates an Observer with given inner observer.
Click to show internal directories.
Click to hide internal directories.